Class Mapping API — SQLAlchemy 1.1 Documentation

Release: 1.1.16 | Release Date: February 16, 2018

# SQLAlchemy 1.1 Documentation

# Class Mapping API

`sqlalchemy.orm.` `mapper`(class_, local_table=None, properties=None, primary_key=None, non_primary=False, inherits=None, inherit_condition=None, inherit_foreign_keys=None, extension=None, order_by=False, always_refresh=False, version_id_col=None, version_id_generator=None, polymorphic_on=None, _polymorphic_map=None, polymorphic_identity=None, concrete=False, with_polymorphic=None, allow_partial_pks=True, batch=True, column_prefix=None, include_properties=None, exclude_properties=None, passive_updates=True, passive_deletes=False, confirm_deleted_rows=True, eager_defaults=False, legacy_is_orphan=False, _compiled_cache_size=100)

  Return a new `Mapper` object.

  This function is typically used behind the scenes via the Declarative extension. When using Declarative, many of the usual `mapper()` arguments are handled by the Declarative extension itself, including `class_`, `local_table`, `properties`, and `inherits`. Other options are passed to `mapper()` using the `__mapper_args__` class variable:

      class MyClass(Base):
          __tablename__ = 'my_table'
          id = Column(Integer, primary_key=True)
          type = Column(String(50))
          alt = Column("some_alt", Integer)

          __mapper_args__ = {
              'polymorphic_on' : type
          }

  Explicit use of `mapper()` is often referred to as classical mapping. The above declarative example is equivalent in classical form to:

      my_table = Table("my_table", metadata,
          Column('id', Integer, primary_key=True),
          Column('type', String(50)),
          Column("some_alt", Integer)
      )

      class MyClass(object):
          pass

      mapper(MyClass, my_table,
          polymorphic_on=my_table.c.type,
          properties={
              'alt':my_table.c.some_alt
          })

  See also

  Classical Mappings - discussion of direct usage of `mapper()`

  - Parameters: | class_ – The class to be mapped. When using Declarative, this argument is automatically passed as the declared class itself. local_table – The `Table` or other selectable to which the class is mapped. May be `None` if this mapper inherits from another mapper using single-table inheritance. When using Declarative, this argument is automatically passed by the extension, based on what is configured via the `__table__` argument or via the `Table` produced as a result of the `__tablename__` and `Column` arguments present. always_refresh – If True, all query operations for this mapped class will overwrite all data within object instances that already exist within the session, erasing any in-memory changes with whatever information was loaded from the database. Usage of this flag is highly discouraged; as an alternative, see the method `Query.populate_existing()`. allow_partial_pks – Defaults to True. Indicates that a composite primary key with some NULL values should be considered as possibly existing within the database. This affects whether a mapper will assign an incoming row to an existing identity, as well as if `Session.merge()` will check the database first for a particular primary key value. A “partial primary key” can occur if one has mapped to an OUTER JOIN, for example. batch – Defaults to `True`, indicating that save operations of multiple entities can be batched together for efficiency. Setting to False indicates that an instance will be fully saved before saving the next instance. This is used in the extremely rare case that a `MapperEvents` listener requires being called in between individual row persistence operations. column_prefix – A string which will be prepended to the mapped attribute name when `Column` objects are automatically assigned as attributes to the mapped class. Does not affect explicitly specified column-based properties. See the section Naming All Columns with a Prefix for an example. concrete – If True, indicates this mapper should use concrete table inheritance with its parent mapper. See the section Concrete Table Inheritance for an example. confirm_deleted_rows – defaults to True; when a DELETE occurs of one more rows based on specific primary keys, a warning is emitted when the number of rows matched does not equal the number of rows expected. This parameter may be set to False to handle the case where database ON DELETE CASCADE rules may be deleting some of those rows automatically. The warning may be changed to an exception in a future release. New in version 0.9.4: - added `mapper.confirm_deleted_rows` as well as conditional matched row checking on delete. eager_defaults – if True, the ORM will immediately fetch the value of server-generated default values after an INSERT or UPDATE, rather than leaving them as expired to be fetched on next access. This can be used for event schemes where the server-generated values are needed immediately before the flush completes. By default, this scheme will emit an individual `SELECT` statement per row inserted or updated, which note can add significant performance overhead. However, if the target database supports RETURNING, the default values will be returned inline with the INSERT or UPDATE statement, which can greatly enhance performance for an application that needs frequent access to just-generated server defaults. Changed in version 0.9.0: The `eager_defaults` option can now make use of RETURNING for backends which support it. exclude_properties – A list or set of string column names to be excluded from mapping. See Mapping a Subset of Table Columns for an example. extension – A `MapperExtension` instance or list of `MapperExtension` instances which will be applied to all operations by this `Mapper`. Deprecated. Please see `MapperEvents`. include_properties – An inclusive list or set of string column names to map. See Mapping a Subset of Table Columns for an example. inherits – A mapped class or the corresponding `Mapper` of one indicating a superclass to which this `Mapper` should inherit from. The mapped class here must be a subclass of the other mapper’s class. When using Declarative, this argument is passed automatically as a result of the natural class hierarchy of the declared classes. See also Mapping Class Inheritance Hierarchies inherit_condition – For joined table inheritance, a SQL expression which will define how the two tables are joined; defaults to a natural join between the two tables. inherit_foreign_keys – When `inherit_condition` is used and the columns present are missing a `ForeignKey` configuration, this parameter can be used to specify which columns are “foreign”. In most cases can be left as `None`. legacy_is_orphan – Boolean, defaults to `False`. When `True`, specifies that “legacy” orphan consideration is to be applied to objects mapped by this mapper, which means that a pending (that is, not persistent) object is auto-expunged from an owning `Session` only when it is de-associated from all parents that specify a `delete-orphan` cascade towards this mapper. The new default behavior is that the object is auto-expunged when it is de-associated with any of its parents that specify `delete-orphan` cascade. This behavior is more consistent with that of a persistent object, and allows behavior to be consistent in more scenarios independently of whether or not an orphanable object has been flushed yet or not. See the change note and example at The consideration of a “pending” object as an “orphan” has been made more aggressive for more detail on this change. New in version 0.8: - the consideration of a pending object as an “orphan” has been modified to more closely match the behavior as that of persistent objects, which is that the object is expunged from the `Session` as soon as it is de-associated from any of its orphan-enabled parents. Previously, the pending object would be expunged only if de-associated from all of its orphan-enabled parents. The new flag `legacy_is_orphan` is added to `orm.mapper()` which re-establishes the legacy behavior. non_primary – Specify that this `Mapper` is in addition to the “primary” mapper, that is, the one used for persistence. The `Mapper` created here may be used for ad-hoc mapping of the class to an alternate selectable, for loading only. `Mapper.non_primary` is not an often used option, but is useful in some specific `relationship()` cases. See also Relationship to Non Primary Mapper order_by – A single `Column` or list of `Column` objects for which selection operations should use as the default ordering for entities. By default mappers have no pre-defined ordering. Deprecated since version 1.1: The `Mapper.order_by` parameter is deprecated. Use `Query.order_by()` to determine the ordering of a result set. passive_deletes – Indicates DELETE behavior of foreign key columns when a joined-table inheritance entity is being deleted. Defaults to `False` for a base mapper; for an inheriting mapper, defaults to `False` unless the value is set to `True` on the superclass mapper. When `True`, it is assumed that ON DELETE CASCADE is configured on the foreign key relationships that link this mapper’s table to its superclass table, so that when the unit of work attempts to delete the entity, it need only emit a DELETE statement for the superclass table, and not this table. When `False`, a DELETE statement is emitted for this mapper’s table individually. If the primary key attributes local to this table are unloaded, then a SELECT must be emitted in order to validate these attributes; note that the primary key columns of a joined-table subclass are not part of the “primary key” of the object as a whole. Note that a value of `True` is always forced onto the subclass mappers; that is, it’s not possible for a superclass to specify passive_deletes without this taking effect for all subclass mappers. New in version 1.1. See also Using Passive Deletes - description of similar feature as used with `relationship()` `mapper.passive_updates` - supporting ON UPDATE CASCADE for joined-table inheritance mappers passive_updates – Indicates UPDATE behavior of foreign key columns when a primary key column changes on a joined-table inheritance mapping. Defaults to `True`. When True, it is assumed that ON UPDATE CASCADE is configured on the foreign key in the database, and that the database will handle propagation of an UPDATE from a source column to dependent columns on joined-table rows. When False, it is assumed that the database does not enforce referential integrity and will not be issuing its own CASCADE operation for an update. The unit of work process will emit an UPDATE statement for the dependent columns during a primary key change. See also Mutable Primary Keys / Update Cascades - description of a similar feature as used with `relationship()` `mapper.passive_deletes` - supporting ON DELETE CASCADE for joined-table inheritance mappers polymorphic_on – Specifies the column, attribute, or SQL expression used to determine the target class for an incoming row, when inheriting classes are present. This value is commonly a `Column` object that’s present in the mapped `Table`:
        class Employee(Base):
            __tablename__ = 'employee'

            id = Column(Integer, primary_key=True)
            discriminator = Column(String(50))

            __mapper_args__ = {
                "polymorphic_on":discriminator,
                "polymorphic_identity":"employee"
            }
    It may also be specified as a SQL expression, as in this example where we use the `case()` construct to provide a conditional approach:
        class Employee(Base):
            __tablename__ = 'employee'

            id = Column(Integer, primary_key=True)
            discriminator = Column(String(50))

            __mapper_args__ = {
                "polymorphic_on":case([
                    (discriminator == "EN", "engineer"),
                    (discriminator == "MA", "manager"),
                ], else_="employee"),
                "polymorphic_identity":"employee"
            }
    It may also refer to any attribute configured with `column_property()`, or to the string name of one:
        class Employee(Base):
            __tablename__ = 'employee'

            id = Column(Integer, primary_key=True)
            discriminator = Column(String(50))
            employee_type = column_property(
                case([
                    (discriminator == "EN", "engineer"),
                    (discriminator == "MA", "manager"),
                ], else_="employee")
            )

            __mapper_args__ = {
                "polymorphic_on":employee_type,
                "polymorphic_identity":"employee"
            }
    Changed in version 0.7.4: `polymorphic_on` may be specified as a SQL expression, or refer to any attribute configured with `column_property()`, or to the string name of one. When setting `polymorphic_on` to reference an attribute or expression that’s not present in the locally mapped `Table`, yet the value of the discriminator should be persisted to the database, the value of the discriminator is not automatically set on new instances; this must be handled by the user, either through manual means or via event listeners. A typical approach to establishing such a listener looks like:
        from sqlalchemy import event
        from sqlalchemy.orm import object_mapper

        @event.listens_for(Employee, "init", propagate=True)
        def set_identity(instance, *arg, **kw):
            mapper = object_mapper(instance)
            instance.discriminator = mapper.polymorphic_identity
  - Where above, we assign the value of `polymorphic_identity` for the mapped class to the `discriminator` attribute, thus persisting the value to the `discriminator` column in the database. Warning Currently, only one discriminator column may be set, typically on the base-most class in the hierarchy. “Cascading” polymorphic columns are not yet supported. See also Mapping Class Inheritance Hierarchies polymorphic_identity – Specifies the value which identifies this particular class as returned by the column expression referred to by the `polymorphic_on` setting. As rows are received, the value corresponding to the `polymorphic_on` column expression is compared to this value, indicating which subclass should be used for the newly reconstructed object. properties – A dictionary mapping the string names of object attributes to `MapperProperty` instances, which define the persistence behavior of that attribute. Note that `Column` objects present in the mapped `Table` are automatically placed into `ColumnProperty` instances upon mapping, unless overridden. When using Declarative, this argument is passed automatically, based on all those `MapperProperty` instances declared in the declared class body. primary_key – A list of `Column` objects which define the primary key to be used against this mapper’s selectable unit. This is normally simply the primary key of the `local_table`, but can be overridden here. version_id_col – A `Column` that will be used to keep a running version id of rows in the table. This is used to detect concurrent updates or the presence of stale data in a flush. The methodology is to detect if an UPDATE statement does not match the last known version id, a `StaleDataError` exception is thrown. By default, the column must be of `Integer` type, unless `version_id_generator` specifies an alternative version generator. See also Configuring a Version Counter - discussion of version counting and rationale. version_id_generator – Define how new version ids should be generated. Defaults to `None`, which indicates that a simple integer counting scheme be employed. To provide a custom versioning scheme, provide a callable function of the form:
        def generate_version(version):
            return next_version
  Alternatively, server-side versioning functions such as triggers, or programmatic versioning schemes outside of the version id generator may be used, by specifying the value `False`. Please see Server Side Version Counters for a discussion of important points when using this option. New in version 0.9.0: `version_id_generator` supports server-side version number generation. See also Custom Version Counters / Types Server Side Version Counters with_polymorphic – A tuple in the form `(<classes>, <selectable>)` indicating the default style of “polymorphic” loading, that is, which tables are queried at once. <classes> is any single or list of mappers and/or classes indicating the inherited classes that should be loaded at once. The special value `'*'` may be used to indicate all descending classes should be loaded immediately. The second tuple argument <selectable> indicates a selectable that will be used to query for multiple classes. See also Using with_polymorphic - discussion of polymorphic querying techniques.

`sqlalchemy.orm.` `object_mapper`(instance)

  Given an object, return the primary Mapper associated with the object instance.

  Raises `sqlalchemy.orm.exc.UnmappedInstanceError` if no mapping is configured.

  This function is available via the inspection system as:

      inspect(instance).mapper

  Using the inspection system will raise `sqlalchemy.exc.NoInspectionAvailable` if the instance is not part of a mapping.

`sqlalchemy.orm.` `class_mapper`(class_, configure=True)

  Given a class, return the primary `Mapper` associated with the key.

  Raises `UnmappedClassError` if no mapping is configured on the given class, or `ArgumentError` if a non-class object is passed.

  Equivalent functionality is available via the `inspect()` function as:

      inspect(some_mapped_class)

  Using the inspection system will raise `sqlalchemy.exc.NoInspectionAvailable` if the class is not mapped.

`sqlalchemy.orm.` `configure_mappers`()

  Initialize the inter-mapper relationships of all mappers that have been constructed thus far.

  This function can be called any number of times, but in most cases is invoked automatically, the first time mappings are used, as well as whenever mappings are used and additional not-yet-configured mappers have been constructed.

  Points at which this occur include when a mapped class is instantiated into an instance, as well as when the `Session.query()` method is used.

  The `configure_mappers()` function provides several event hooks that can be used to augment its functionality. These methods include:

  - `MapperEvents.before_configured()` - called once before `configure_mappers()` does any work; this can be used to establish additional options, properties, or related mappings before the operation proceeds.
  - `MapperEvents.mapper_configured()` - called as each indivudal `Mapper` is configured within the process; will include all mapper state except for backrefs set up by other mappers that are still to be configured.
  - `MapperEvents.after_configured()` - called once after `configure_mappers()` is complete; at this stage, all `Mapper` objects that are known to SQLAlchemy will be fully configured. Note that the calling application may still have other mappings that haven’t been produced yet, such as if they are in modules as yet unimported.

`sqlalchemy.orm.` `clear_mappers`()

  Remove all mappers from all classes.

  This function removes all instrumentation from classes and disposes of their associated mappers. Once called, the classes are unmapped and can be later re-mapped with new mappers.

  `clear_mappers()` is not for normal use, as there is literally no valid usage for it outside of very specific testing scenarios. Normally, mappers are permanent structural components of user-defined classes, and are never discarded independently of their class. If a mapped class itself is garbage collected, its mapper is automatically disposed of as well. As such, `clear_mappers()` is only for usage in test suites that re-use the same classes with different mappings, which is itself an extremely rare use case - the only such use case is in fact SQLAlchemy’s own test suite, and possibly the test suites of other ORM extension libraries which intend to test various combinations of mapper construction upon a fixed set of classes.

`sqlalchemy.orm.util.` `identity_key`(*args, **kwargs)

  Generate “identity key” tuples, as are used as keys in the `Session.identity_map` dictionary.

  This function has several call styles:

  - `identity_key(class, ident)`

    This form receives a mapped class and a primary key scalar or tuple as an argument.

    E.g.:

        >>> identity_key(MyClass, (1, 2))
        (<class '__main__.MyClass'>, (1, 2))

    param class: | mapped class (must be a positional argument)
    param ident: | primary key, may be a scalar or tuple argument.

  - `identity_key(instance=instance)`

    This form will produce the identity key for a given instance. The instance need not be persistent, only that its primary key attributes are populated (else the key will contain `None` for those missing values).

    E.g.:

        >>> instance = MyClass(1, 2)
        >>> identity_key(instance=instance)
        (<class '__main__.MyClass'>, (1, 2))

    In this form, the given instance is ultimately run though `Mapper.identity_key_from_instance()`, which will have the effect of performing a database check for the corresponding row if the object is expired.

    param instance: | object instance (must be given as a keyword arg)

  - `identity_key(class, row=row)`

    This form is similar to the class/tuple form, except is passed a database result row as a `RowProxy` object.

    E.g.:

        >>> row = engine.execute("select * from table where a=1 and b=2").first()
        >>> identity_key(MyClass, row=row)
        (<class '__main__.MyClass'>, (1, 2))

    param class: | mapped class (must be a positional argument)
    param row: | `RowProxy` row returned by a `ResultProxy` (must be given as a keyword arg)

`sqlalchemy.orm.util.` `polymorphic_union`(table_map, typecolname, aliasname='p_union', cast_nulls=True)

  Create a `UNION` statement used by a polymorphic mapper.

  See Concrete Table Inheritance for an example of how this is used.

  Parameters: | table_map – mapping of polymorphic identities to `Table` objects. typecolname – string name of a “discriminator” column, which will be derived from the query, producing the polymorphic identity for each row. If `None`, no polymorphic discriminator is generated. aliasname – name of the `alias()` construct generated. cast_nulls – if True, non-existent columns, which are represented as labeled NULLs, will be passed into CAST. This is a legacy behavior that is problematic on some backends such as Oracle - in which case it can be set to False.

class `sqlalchemy.orm.mapper.` `Mapper`(class_, local_table=None, properties=None, primary_key=None, non_primary=False, inherits=None, inherit_condition=None, inherit_foreign_keys=None, extension=None, order_by=False, always_refresh=False, version_id_col=None, version_id_generator=None, polymorphic_on=None, _polymorphic_map=None, polymorphic_identity=None, concrete=False, with_polymorphic=None, allow_partial_pks=True, batch=True, column_prefix=None, include_properties=None, exclude_properties=None, passive_updates=True, passive_deletes=False, confirm_deleted_rows=True, eager_defaults=False, legacy_is_orphan=False, _compiled_cache_size=100)

  Bases: `sqlalchemy.orm.base.InspectionAttr`

  Define the correlation of class attributes to database table columns.

  The `Mapper` object is instantiated using the `mapper()` function. For information about instantiating new `Mapper` objects, see that function’s documentation.

  When `mapper()` is used explicitly to link a user defined class with table metadata, this is referred to as classical mapping. Modern SQLAlchemy usage tends to favor the `sqlalchemy.ext.declarative` extension for class configuration, which makes usage of `mapper()` behind the scenes.

  Given a particular class known to be mapped by the ORM, the `Mapper` which maintains it can be acquired using the `inspect()` function:

      from sqlalchemy import inspect

      mapper = inspect(MyClass)

  A class which was mapped by the `sqlalchemy.ext.declarative` extension will also have its mapper available via the `__mapper__` attribute.

  `__init__`(class_, local_table=None, properties=None, primary_key=None, non_primary=False, inherits=None, inherit_condition=None, inherit_foreign_keys=None, extension=None, order_by=False, always_refresh=False, version_id_col=None, version_id_generator=None, polymorphic_on=None, _polymorphic_map=None, polymorphic_identity=None, concrete=False, with_polymorphic=None, allow_partial_pks=True, batch=True, column_prefix=None, include_properties=None, exclude_properties=None, passive_updates=True, passive_deletes=False, confirm_deleted_rows=True, eager_defaults=False, legacy_is_orphan=False, _compiled_cache_size=100)

    Construct a new `Mapper` object.

    This constructor is mirrored as a public API function; see `mapper()` for a full usage and argument description.

  `add_properties`(dict_of_properties)

    Add the given dictionary of properties to this mapper, using add_property.

  `add_property`(key, prop)

    Add an individual MapperProperty to this mapper.

    If the mapper has not been configured yet, just adds the property to the initial properties dictionary sent to the constructor. If this Mapper has already been configured, then the given MapperProperty is configured immediately.

  `all_orm_descriptors`

    A namespace of all `InspectionAttr` attributes associated with the mapped class.

    These attributes are in all cases Python descriptors associated with the mapped class or its superclasses.

    This namespace includes attributes that are mapped to the class as well as attributes declared by extension modules. It includes any Python descriptor type that inherits from `InspectionAttr`. This includes `QueryableAttribute`, as well as extension types such as `hybrid_property`, `hybrid_method` and `AssociationProxy`.

    To distinguish between mapped attributes and extension attributes, the attribute `InspectionAttr.extension_type` will refer to a constant that distinguishes between different extension types.

    When dealing with a `QueryableAttribute`, the `QueryableAttribute.property` attribute refers to the `MapperProperty` property, which is what you get when referring to the collection of mapped properties via `Mapper.attrs`.

    Warning

    The `Mapper.all_orm_descriptors` accessor namespace is an instance of `OrderedProperties`. This is a dictionary-like object which includes a small number of named methods such as `OrderedProperties.items()` and `OrderedProperties.values()`. When accessing attributes dynamically, favor using the dict-access scheme, e.g. `mapper.all_orm_descriptors[somename]` over `getattr(mapper.all_orm_descriptors, somename)` to avoid name collisions.

    New in version 0.8.0.

    See also

    `Mapper.attrs`

  `attrs`

    A namespace of all `MapperProperty` objects associated this mapper.

    This is an object that provides each property based on its key name. For instance, the mapper for a `User` class which has `User.name` attribute would provide `mapper.attrs.name`, which would be the `ColumnProperty` representing the `name` column. The namespace object can also be iterated, which would yield each `MapperProperty`.

    `Mapper` has several pre-filtered views of this attribute which limit the types of properties returned, inclding `synonyms`, `column_attrs`, `relationships`, and `composites`.

    Warning

    The `Mapper.attrs` accessor namespace is an instance of `OrderedProperties`. This is a dictionary-like object which includes a small number of named methods such as `OrderedProperties.items()` and `OrderedProperties.values()`. When accessing attributes dynamically, favor using the dict-access scheme, e.g. `mapper.attrs[somename]` over `getattr(mapper.attrs, somename)` to avoid name collisions.

    See also

    `Mapper.all_orm_descriptors`

  `base_mapper` = None

    The base-most `Mapper` in an inheritance chain.

    In a non-inheriting scenario, this attribute will always be this `Mapper`. In an inheritance scenario, it references the `Mapper` which is parent to all other `Mapper` objects in the inheritance chain.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `c` = None

    A synonym for `columns`.

  `cascade_iterator`(type_, state, halt_on=None)

    Iterate each element and its mapper in an object graph, for all relationships that meet the given cascade rule.

    Parameters: | type_ – The name of the cascade rule (i.e. `"save-update"`, `"delete"`, etc.). Note the `"all"` cascade is not accepted here. For a generic object traversal function, see How do I walk all objects that are related to a given object?. state – The lead InstanceState. child items will be processed per the relationships defined for this object’s mapper.
    Returns: | the method yields individual object instances.

    See also

    Cascades

    How do I walk all objects that are related to a given object? - illustrates a generic function to traverse all objects without relying on cascades.

  `class_` = None

    The Python class which this `Mapper` maps.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `class_manager` = None

    The `ClassManager` which maintains event listeners and class-bound descriptors for this `Mapper`.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `column_attrs`

    Return a namespace of all `ColumnProperty` properties maintained by this `Mapper`.

    See also

    `Mapper.attrs` - namespace of all `MapperProperty` objects.

  `columns` = None

    A collection of `Column` or other scalar expression objects maintained by this `Mapper`.

    The collection behaves the same as that of the `c` attribute on any `Table` object, except that only those columns included in this mapping are present, and are keyed based on the attribute name defined in the mapping, not necessarily the `key` attribute of the `Column` itself. Additionally, scalar expressions mapped by `column_property()` are also present here.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `common_parent`(other)

    Return true if the given mapper shares a common inherited parent as this mapper.

  `composites`

    Return a namespace of all `CompositeProperty` properties maintained by this `Mapper`.

    See also

    `Mapper.attrs` - namespace of all `MapperProperty` objects.

  `concrete` = None

    Represent `True` if this `Mapper` is a concrete inheritance mapper.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `configured` = None

    Represent `True` if this `Mapper` has been configured.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

    See also

    `configure_mappers()`.

  `entity`

    Part of the inspection API.

    Returns self.class_.

  `get_property`(key, _configure_mappers=True)

    return a MapperProperty associated with the given key.

  `get_property_by_column`(column)

    Given a `Column` object, return the `MapperProperty` which maps this column.

  `identity_key_from_instance`(instance)

    Return the identity key for the given instance, based on its primary key attributes.

    If the instance’s state is expired, calling this method will result in a database check to see if the object has been deleted. If the row no longer exists, `ObjectDeletedError` is raised.

    This value is typically also found on the instance state under the attribute name key.

  `identity_key_from_primary_key`(primary_key)

    Return an identity-map key for use in storing/retrieving an item from an identity map.

    Parameters: | primary_key – A list of values indicating the identifier.

  `identity_key_from_row`(row, adapter=None)

    Return an identity-map key for use in storing/retrieving an item from the identity map.

    Parameters: | row – A `RowProxy` instance. The columns which are mapped by this `Mapper` should be locatable in the row, preferably via the `Column` object directly (as is the case when a `select()` construct is executed), or via string names of the form `<tablename>_<colname>`.

  `inherits` = None

    References the `Mapper` which this `Mapper` inherits from, if any.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `is_mapper` = True

    Part of the inspection API.

  `isa`(other)

    Return True if the this mapper inherits from the given mapper.

  `iterate_properties`

    return an iterator of all MapperProperty objects.

  `local_table` = None

    The `Selectable` which this `Mapper` manages.

    Typically is an instance of `Table` or `Alias`. May also be `None`.

    The “local” table is the selectable that the `Mapper` is directly responsible for managing from an attribute access and flush perspective. For non-inheriting mappers, the local table is the same as the “mapped” table. For joined-table inheritance mappers, local_table will be the particular sub-table of the overall “join” which this `Mapper` represents. If this mapper is a single-table inheriting mapper, local_table will be `None`.

    See also

    `mapped_table`.

  `mapped_table` = None

    The `Selectable` to which this `Mapper` is mapped.

    Typically an instance of `Table`, `Join`, or `Alias`.

    The “mapped” table is the selectable that the mapper selects from during queries. For non-inheriting mappers, the mapped table is the same as the “local” table. For joined-table inheritance mappers, mapped_table references the full `Join` representing full rows for this particular subclass. For single-table inheritance mappers, mapped_table references the base table.

    See also

    `local_table`.

  `mapper`

    Part of the inspection API.

    Returns self.

  `non_primary` = None

    Represent `True` if this `Mapper` is a “non-primary” mapper, e.g. a mapper that is used only to selet rows but not for persistence management.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `polymorphic_identity` = None

    Represent an identifier which is matched against the `polymorphic_on` column during result row loading.

    Used only with inheritance, this object can be of any type which is comparable to the type of column represented by `polymorphic_on`.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `polymorphic_iterator`()

    Iterate through the collection including this mapper and all descendant mappers.

    This includes not just the immediately inheriting mappers but all their inheriting mappers as well.

    To iterate through an entire hierarchy, use `mapper.base_mapper.polymorphic_iterator()`.

  `polymorphic_map` = None

    A mapping of “polymorphic identity” identifiers mapped to `Mapper` instances, within an inheritance scenario.

    The identifiers can be of any type which is comparable to the type of column represented by `polymorphic_on`.

    An inheritance chain of mappers will all reference the same polymorphic map object. The object is used to correlate incoming result rows to target mappers.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `polymorphic_on` = None

    The `Column` or SQL expression specified as the `polymorphic_on` argument for this `Mapper`, within an inheritance scenario.

    This attribute is normally a `Column` instance but may also be an expression, such as one derived from `cast()`.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `primary_key` = None

    An iterable containing the collection of `Column` objects which comprise the ‘primary key’ of the mapped table, from the perspective of this `Mapper`.

    This list is against the selectable in `mapped_table`. In the case of inheriting mappers, some columns may be managed by a superclass mapper. For example, in the case of a `Join`, the primary key is determined by all of the primary key columns across all tables referenced by the `Join`.

    The list is also not necessarily the same as the primary key column collection associated with the underlying tables; the `Mapper` features a `primary_key` argument that can override what the `Mapper` considers as primary key columns.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `primary_key_from_instance`(instance)

    Return the list of primary key values for the given instance.

    If the instance’s state is expired, calling this method will result in a database check to see if the object has been deleted. If the row no longer exists, `ObjectDeletedError` is raised.

  `primary_mapper`()

    Return the primary mapper corresponding to this mapper’s class key (class).

  `relationships`

    A namespace of all `RelationshipProperty` properties maintained by this `Mapper`.

    Warning

    the `Mapper.relationships` accessor namespace is an instance of `OrderedProperties`. This is a dictionary-like object which includes a small number of named methods such as `OrderedProperties.items()` and `OrderedProperties.values()`. When accessing attributes dynamically, favor using the dict-access scheme, e.g. `mapper.relationships[somename]` over `getattr(mapper.relationships, somename)` to avoid name collisions.

    See also

    `Mapper.attrs` - namespace of all `MapperProperty` objects.

  `selectable`

    The `select()` construct this `Mapper` selects from by default.

    Normally, this is equivalent to `mapped_table`, unless the `with_polymorphic` feature is in use, in which case the full “polymorphic” selectable is returned.

  `self_and_descendants`

    The collection including this mapper and all descendant mappers.

    This includes not just the immediately inheriting mappers but all their inheriting mappers as well.

  `single` = None

    Represent `True` if this `Mapper` is a single table inheritance mapper.

    `local_table` will be `None` if this flag is set.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `synonyms`

    Return a namespace of all `SynonymProperty` properties maintained by this `Mapper`.

    See also

    `Mapper.attrs` - namespace of all `MapperProperty` objects.

  `tables` = None

    An iterable containing the collection of `Table` objects which this `Mapper` is aware of.

    If the mapper is mapped to a `Join`, or an `Alias` representing a `Select`, the individual `Table` objects that comprise the full construct will be represented here.

    This is a read only attribute determined during mapper construction. Behavior is undefined if directly modified.

  `validators` = None

    An immutable dictionary of attributes which have been decorated using the `validates()` decorator.

    The dictionary contains string attribute names as keys mapped to the actual validation method.

  `with_polymorphic_mappers`

    The list of `Mapper` objects included in the default “polymorphic” query.